function [R,wage,mvec,Kvec,zeta,aGrid,aDist] = getStationaryEq3(beta,P,p,phi,A,alpha,delta,aMultiple,Ngrid)
% compute stationary equilibrium of perpetual youth Aiyagari economy with
% log utility, minimum consumption, Cobb-Douglas production function

% beta:     vector of discount factor
% P:        transition probability matrix
% p:        birth-death probability
% phi:      minimum consumption as fraction of wage income
% A:        TFP
% alpha:    capital share
% delta:    capital depreciation rate

%% some error checking

if any(beta<0)
    error('beta must be positive')
end

S = size(P,1); % number of states
if length(beta) ~= S
    error('beta must have same length as beta')
end

if size(beta,1) < size(beta,2)
    error('beta must be a column vector')
end

if (p<=0)||(p>=1)
    error('p must be between 0 and 1')
end

if (alpha<=0)||(alpha>=1)
    error('alpha must be between 0 and 1')
end

if (delta<0)||(delta>1)
    error('delta must be between 0 and 1')
end

[v,~] = eigs(P',1);
pi = v/sum(v); % stationary distribution
if (1-p)*eigs(diag(beta)*P) >= 1
    error('no solution - beta too large')
end

%% compute stationary equilibrium
Rlb = max(1-p,1-delta); % lower bound for gross risk-free rate

B = (eye(S) - (1-p)*diag(beta)*P)\ones(S,1);
mvec = 1./B;
M = diag(mvec); % diagonal matrix of MPC
C = P'*(eye(S)-M);

temp = C*pi - (1-p)*eigs(C,1)*pi;
if any(temp<0)
    error('nonexistence')
end

Rub = 1/eigs(C,1); % upper bound for gross risk free rate

% define equilibrium condition
func = @(R)EqCond3(R,beta,P,p,phi,A,alpha,delta);

% do grid search to select good initial value
N = 101;
Rgrid = linspace(Rlb + 1e-6,Rub - 1e-6,N)';
fval = zeros(N,1);
for n=1:N
    fval(n) = func(Rgrid(n));
end

n = find(fval > 0,1,'last');
Rlb = Rgrid(n);
Rub = Rgrid(n+1);

R = fzero(func,[Rlb,Rub]); % equilibrium gross risk-free rate

wage = A^(1/(1-alpha))*(1-alpha)*((R-1+delta)/alpha)^(alpha/(alpha-1)); % wage
yEff = (1-phi)*wage; % disposable income
h = yEff/(1-(1-p)/R); % human capital

Kvec = h*((eye(S) - R*C)\(C*pi - ((1-p)/R)*pi)); % vector of aggregate capital supply

Reff = R/(1-p); % effective gross risk-free rate
D = diag(Reff*(1-mvec));
lambda = @(z)((1-p)*eigs(P*(D.^z),1)-1);

[b,s] = max(B);
temp = Reff*max(1-1/b);
zetaUB = -log((1-p)*P(s,s))/log(temp);
zeta = fzero(lambda,[1 zetaUB]);

if nargin < 8 % don't compute stationary distribution
    aGrid = [];
    aDist = [];
    return
end

%% compute stationary distribution using Pareto extrapolation

betaBar = dot(pi,beta); % average discount factor
K_RA = ((1/(betaBar*(1-p))-1+delta)/(A*alpha))^(1/(alpha-1));
aMed = K_RA; % aggregate asset used as median point
aMax = K_RA*aMultiple;
aMin = -h; % minimum financial wealth

s = (aMed^2-aMin*aMax)/(aMin+aMax-2*aMed); % shift parameter
temp = linspace(log(aMin+s),log(aMax+s),Ngrid+1); % even grid in log scale
aGrid = exp(temp(2:end))-s; % asset grid
Nmed = floor(Ngrid/2);
aGrid(1:Nmed) = linspace(aGrid(1),aGrid(Nmed),Nmed);

aNextMat = Reff*(bsxfun(@minus,(1-mvec)*aGrid,h*mvec)+(1-phi)*wage);
[~,stationaryDistribution] = getQ(P,1,1-p,0,aGrid,aNextMat,Reff*(1-mvec),zeta);
aDist = sum(reshape(stationaryDistribution,Ngrid,S),2);

end

