%% evaluate solution accuracy in partial equilibrium with various grids
clear;
close all
clc;

test_getStationaryEq3 % run this code to get parameters

labor = 1; % labor supply
yEff = wage*(1-phi).*labor; % effective disposable income
K = sum(Kvec); % aggregate capital
A_agg = R*K; % aggregate financial wealth
betaBar = dot(pi,beta);
K_RA = ((1/(betaBar*(1-p))-1+delta)/(A*alpha))^(1/(alpha-1)); % aggregate capital in representative-agent model
Nanal = 2000; % number of grid points when computing highly accurate wealth distribution for plotting results
% (**Don't set it too large - it will crash! 3000 is maximum**)
aMaxAnal = K_RA*1e6; % truncation point for analytical results
aMin = -wage*(1-phi)*min(labor)/(1-(1-p)/R); % natural borrowing limit
aMed = K_RA; % median grid point
aGridAnal = expGrid(aMin,aMaxAnal,aMed,Nanal); % exponential grid on [aMin,aMax] with median point 0
Nmed = floor(Nanal/2);
aGridAnal(1:Nmed) = linspace(aMin,aGridAnal(Nmed),Nmed); % replace bottom half by even-space grid

flagGrid = 1;
% 0: evenly-spaced grid, not in paper
% 1: affine-exponential with fixed N
% 2: affine-exponential with fixed density
if flagGrid == 0
    aMaxVec = K_RA*[5 10 20];
    Nvec = [100 200 400];
elseif flagGrid == 1
    aMaxlog10 = [1:6];
    aMaxVec = K_RA*10.^aMaxlog10;
    Nvec = [10 100 1000]; % number of grid points
elseif flagGrid == 2
    aMaxlog10 = [1:1:6];
    aMaxVec = K_RA*10.^aMaxlog10;
    Nvec = [10 20 30];
elseif flagGrid == 3
    Navec = [Nmed:100:Nanal];
    aMaxVec = aGridAnal(Navec);
    aMaxVec = aMaxVec(aMaxVec > 2*aMed - aMin);
    aMaxlog10 = log10(aMaxVec/K_RA);
    Nvec = 1;
end

Na = length(aMaxVec);
NN = length(Nvec);

%% compute top wealth shares from semi-analytical solution using highly accurate exponential grid

log10LB = -4; % lower bound to plot results in log10
p_rich = 0.0006; % fraction of billionaires
topProb = [p_rich/100 10.^[-4:0.5:0]];
J = length(topProb);
temp2 = zeros(Na,2*J); % store top wealth shares

Reff = R/(1-p);
h = yEff/(1-1/Reff); % human capital
aNextMatAnal = Reff*(bsxfun(@minus,(1-mvec)*aGridAnal,h*mvec)+(1-phi)*wage);
[~,stationaryDistribution] = getQ(P,1,1-p,0,aGridAnal,aNextMatAnal,Reff*(1-mvec),zeta);
aDistAnal = sum(reshape(stationaryDistribution,Nanal,S),2);
cCDFAnal = max(1-cumsum(aDistAnal),0);
% top wealth shares
topShareAnal = getTopShares(topProb,aGridAnal,aDistAnal,zeta);

% compute fraction of wealth left out by truncation
AAnal = dot(aDistAnal,aGridAnal) + (1/(zeta-1))*aDistAnal(end)*aGridAnal(end); % aggregate wealth
temp = aDistAnal.*aGridAnal';
temp = cumsum(temp);
AoutShare = 1 - temp/AAnal;

aTheory = [10^3 10^4 10^5];
sTheory = 1000*aTheory.^(1-zeta);

figure
loglog(aGridAnal(1:end-1)/K_RA,100*AoutShare(1:end-1),'LineStyle','-','Color',c1); hold on
loglog(aTheory,sTheory,'--k','LineWidth',1); hold off
text(aTheory(2),1.2*sTheory(2),'Slope = $1-\zeta$')
xlim([10^1 10^6])
xlabel('Truncation point (relative to typical scale)')
ylabel('Wealth share outside grid ($\%$)')
legend('Wealth share outside grid')
xticks([10 10^2 10^3 10^4 10^5 10^6])
yticks([1 2 5 10 20 40])

%save figure in pdf format
fig = gcf;
fig.PaperPositionMode = 'auto';
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
print(fig,'fig_AnalyticalWealthShareOut','-dpdf')

ATruncMat = zeros(Na,NN);
AParetoMat = zeros(Na,NN);
APoorman1Mat = zeros(Na,NN);
APoorman2Mat = zeros(Na,NN);
nGridBottomVec = zeros(Na,1);

for i = 1:Na
    aMax = aMaxVec(i); % truncation point
    for j=1:NN
    % construct grid  
    if flagGrid == 0 % evenly-spaced grid
        N = Nvec(j);
        temp = linspace(aMin,aMax,N+1); % evenly spaced grid
        aGrid = temp(2:end); % exclude 0 because absorbing state
    elseif flagGrid == 1 % affine-exponential grid
        N = Nvec(j);
        aGrid = expGrid(aMin,aMax,aMed,N);
        Nmed = floor(N/2);
        aGrid(1:Nmed) = linspace(aMin,aGrid(Nmed),Nmed); % replace bottom half by even-space grid
    elseif flagGrid == 2
        N = Nvec(j)*ceil(log10(aMax/K_RA));
        aGrid = expGrid(aMin,aMax,aMed,N);
        Nmed = floor(N/2);
        aGrid(1:Nmed) = linspace(aMin,aGrid(Nmed),Nmed);
    elseif flagGrid == 3
        N = sum(aGridAnal <= aMaxVec(i));
        aGrid = aGridAnal(1:N);
    end
    nGridBottomVec(i) = sum(aGrid <= 0); % number of grid points in bottom
        % compute stationary distribution with/without Pareto extrapolation
        aNextMat = Reff*(bsxfun(@minus,(1-mvec)*aGrid,h*mvec)+(1-phi)*wage); % law of motion for asset
        tic
        [~,statDistPE] = getQ(P,1,1-p,0,aGrid,aNextMat,Reff*(1-mvec),zeta); % Pareto extrapolation
        [~,statDistTrunc] = getQ(P,1,1-p,0,aGrid,aNextMat,Reff*(1-mvec),10000); % Truncation
        toc
        aDistPE = sum(reshape(statDistPE,N,S),2);
        aDistTrunc = sum(reshape(statDistTrunc,N,S),2);
        ATruncMat(i,j) = dot(aDistTrunc,aGrid);
        AParetoMat(i,j) = dot(aDistPE,aGrid) + (1/(zeta-1))*aDistPE(end)*aGrid(end);
        APoorman1Mat(i,j) = dot(aDistPE,aGrid);
        APoorman2Mat(i,j) = dot(aDistTrunc,aGrid) + (1/(zeta-1))*aDistTrunc(end)*aGrid(end);
    end
    % compute stationary distributions using finer grids
    if flagGrid < 2
        Neval = 1e3;
    elseif flagGrid == 2
        Neval = 10*N;
    else
        Neval = N;
    end
    if flagGrid < 3
        aGridEval = expGrid(aMin,aMax,aMed,Neval);
        Nmed = floor(Neval/2);
        aGridEval(1:Nmed) = linspace(aMin,aGridEval(Nmed),Nmed);
        %Neval = length(aGridEval);
        aNextMatEval = Reff*(bsxfun(@minus,(1-mvec)*aGridEval,h*mvec)+(1-phi)*wage); % law of motion for asset
        tic
        [~,statDistPE] = getQ(P,1,1-p,0,aGridEval,aNextMatEval,Reff*(1-mvec),zeta); % Pareto extrapolation
        [~,statDistTrunc] = getQ(P,1,1-p,0,aGridEval,aNextMatEval,Reff*(1-mvec),10000); % Truncation
        toc
    else
        aGridEval = aGrid;
    end
    aDist1 = sum(reshape(statDistPE,Neval,S),2);
    aDist0 = sum(reshape(statDistTrunc,Neval,S),2);
    % compute top wealth shares
    topShare0 = getTopShares(topProb,aGridEval,aDist0);
    topShare1 = getTopShares(topProb,aGridEval,aDist1,zeta);
    % collect results in matrix
    temp2(i,:) = [topShare0' topShare1'];
end

relErrorTruncMat = ATruncMat/A_agg-1;
relErrorParetoMat = AParetoMat/A_agg-1;
relErrorPoorman1Mat = APoorman1Mat/A_agg-1;
relErrorPoorman2Mat = APoorman2Mat/A_agg-1;
relErrorMat = 100*[relErrorTruncMat relErrorParetoMat]
nGridBottomVec

% output results as a LaTeX table
temp = [round(100*relErrorTruncMat,2) round(100*relErrorParetoMat,3)];

matrix2latex(temp,'accuracyPE.tex','format','%6.3f')

if flagGrid == 0
    return
end

topShareTrunc = temp2(:,1:J);
topSharePE = temp2(:,J+1:end);

topProbPlot = [p_rich 0.01 0.1 1 10]; % percentiles to plot results
fignames = {'Billionaire','001','01','1','10'};

for j=1:length(topProbPlot)
    ind = find(topProb == topProbPlot(j)/100); % index to plot
    if isempty(ind) == 0
        figure
        semilogx(10.^aMaxlog10',100*topShareAnal(ind)*ones(Na,1),'ko','LineWidth',1); hold on
        semilogx(10.^aMaxlog10',100*topSharePE(:,ind),'LineStyle','-','Color',c1);
        semilogx(10.^aMaxlog10',100*topShareTrunc(:,ind),'LineStyle','--','Color',c1); hold off
        if j==1
            title('Billionaires')
        else
            title(['Top ' num2str(topProbPlot(j)) '$\%$'])
        end
        xlabel('Truncation point (relative to typical wealth)')
        xlim(10.^[aMaxlog10(1) aMaxlog10(end)])
        ylabel('Wealth share ($\%$)')
        legend('Analytical','Pareto extrapolation','Truncation','Location','SE')
        xticks([10 10^2 10^3 10^4 10^5 10^6])
        
        %save figure in pdf format
        fig = gcf;
        fig.PaperPositionMode = 'auto';
        fig_pos = fig.PaperPosition;
        fig.PaperSize = [fig_pos(3) fig_pos(4)];
        print(fig,['fig_AnalyticalWealthShare' fignames{j} 'PE'],'-dpdf')
    end
end

relErrorBillionaireTrunc = topShareTrunc(:,1)./topShareAnal(1)-1;
relErrorBillionairePE = topSharePE(:,1)./topShareAnal(1)-1;
relErrorBillionaire = 100*[relErrorBillionaireTrunc relErrorBillionairePE]

figure
semilogx(10.^aMaxlog10',100*relErrorParetoMat(:,end),'-','Color',c1); hold on
semilogx(10.^aMaxlog10',100*relErrorTruncMat(:,end),'--','Color',c1);
semilogx(10.^aMaxlog10',0*aMaxlog10','ko','LineWidth',1); hold off
title('Aggregate wealth')
xlabel('Truncation point (relative to typical wealth)')
xlim(10.^[aMaxlog10(1) aMaxlog10(end)])
ylabel('Relative error (\%)')
legend('Pareto extrapolation','Truncation','Location','SE')
xticks([10 10^2 10^3 10^4 10^5 10^6])

%save figure in pdf format
fig = gcf;
fig.PaperPositionMode = 'auto';
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
print(fig,'fig_AnalyticalRelErrorAggW','-dpdf')

figure
semilogx(10.^aMaxlog10',100*relErrorBillionairePE,'-','Color',c1); hold on
semilogx(10.^aMaxlog10',100*relErrorBillionaireTrunc,'--','Color',c1);
semilogx(10.^aMaxlog10',0*aMaxlog10','ko','LineWidth',1); hold off
title('Billionaire wealth')
xlabel('Truncation point (relative to typical wealth)')
xlim(10.^[aMaxlog10(1) aMaxlog10(end)])
ylabel('Relative error (\%)')
legend('Pareto extrapolation','Truncation','Location','SE')
xticks([10 10^2 10^3 10^4 10^5 10^6])

%save figure in pdf format
fig = gcf;
fig.PaperPositionMode = 'auto';
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
print(fig,'fig_AnalyticalRelErrorBillionaireW','-dpdf')

%% Plot APC error
if flagGrid < 2
    return
end

c = mvec*(aMaxVec + h) + phi*wage; % consumption function
APC = bsxfun(@rdivide,c,aMaxVec); % APC
relAPCerror = max(abs(bsxfun(@rdivide,APC,mvec)-1),[],1); % maximum MPC relative error

temp = 10.^aMaxlog10';

% aggregate wealth
figure
hold on
plot(temp,abs(relErrorParetoMat(:,end)),'-','Color',c1);
plot(temp,abs(relErrorTruncMat(:,end)),'--','Color',c1);
plot(temp,relAPCerror','k:');
title('Aggregate wealth')
xlim(10.^[aMaxlog10(1) aMaxlog10(end)])
xticks([10 10^2 10^3 10^4 10^5 10^6])
xlabel('Truncation point (relative to typical wealth)')
ylabel('Relative error')
legend('Pareto extrapolation','Truncation','APC','Location','SW')
set(gca,'xscale','log')
set(gca,'yscale','log')

%save figure in pdf format
fig = gcf;
fig.PaperPositionMode = 'auto';
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
print(fig,'fig_AnalyticalAPCAggW','-dpdf')

% billionaire wealth
figure
hold on
plot(temp,abs(relErrorBillionairePE),'-','Color',c1);
plot(temp,abs(relErrorBillionaireTrunc),'--','Color',c1);
plot(temp,relAPCerror','k:');
title('Billionaire wealth')
xlim(10.^[aMaxlog10(1) aMaxlog10(end)])
xticks([10 10^2 10^3 10^4 10^5 10^6])
xlabel('Truncation point (relative to typical wealth)')
ylabel('Relative error')
legend('Pareto extrapolation','Truncation','APC','Location','SW')
set(gca,'xscale','log')
set(gca,'yscale','log')

%save figure in pdf format
fig = gcf;
fig.PaperPositionMode = 'auto';
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
print(fig,'fig_AnalyticalAPCBillionaireW','-dpdf')

%% Poor man's PE

% aggregate wealth
figure
hold on
plot(temp,abs(relErrorTruncMat(:,end)),'--','Color',c1);
plot(temp,abs(relErrorPoorman1Mat(:,end)),'-^','Color',c2);
plot(temp,abs(relErrorPoorman2Mat(:,end)),'-v','Color',c2);
plot(temp,abs(relErrorParetoMat(:,end)),'-','Color',c1);

title('Aggregate wealth')
xlim(10.^[aMaxlog10(1) aMaxlog10(end)])
xticks([10 10^2 10^3 10^4 10^5 10^6])
xlabel('Truncation point (relative to typical wealth)')
ylabel('Relative error')
legend('Truncation','PE (transition probability only)','PE (aggregation only)','Pareto extrapolation','Location','NE')
set(gca,'xscale','log')
set(gca,'yscale','log')

%save figure in pdf format
fig = gcf;
fig.PaperPositionMode = 'auto';
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
print(fig,'fig_AnalyticalRelErrorAggWDecomposition','-dpdf')
